/* vMac Utility for Gemulator ROM Board Settings

   By Weston Pawlowski <Weston@vMac.org>
   Some portions were written by Darek Michocka
   and adapted to work in this utility.

   For vMac 0.1.9.6 for Win32
   Only Detects MacPlus ROM sets

   vMac is Copyright (C) 1997 by Philip Cummins and The vMac Team
   ALL RIGHTS RESERVED
*/

#include <stdio.h>
#include <memory.h>
#include <dos.h>

typedef unsigned char UBYTE;
typedef unsigned short UWORD;
typedef unsigned long ULONG;

ULONG FlipLong (ULONG Data);
ULONG GetROMChecksum(UWORD Base, UBYTE Socket, UBYTE Length);

void main(void)
{
    int Socket;
    UBYTE ROMFound=0;
    UWORD BaseAddr=0x240;
    ULONG Checksum;

    puts("vMac Utility for Gemulator ROM Board Settings");
    puts("For vMac 0.1.9.6 for Win32. Only detects MacPlus ROM sets.\n");

    puts("vMac is Copyright (C) 1997 by Philip Cummins and The vMac Team");
    puts("ALL RIGHTS RESERVED\n");

    puts("The Gemulator ROM board is available from Emulators Inc.");
    puts("http://www.emulators.com\n\n");

    puts("What is the base address (in hex) of your Gemulator ROM board? (Default = 240)");
    scanf("%X",&BaseAddr);

    puts("\nSearching for MacPlus ROMs...");

    for(Socket=1;Socket<=8;++Socket)
    {
        //Mac Plus ROM sets have 2 ROMs, so Length is 2
        Checksum=GetROMChecksum(BaseAddr, Socket, 2);

        Checksum=FlipLong(Checksum);

        switch(Checksum)
        {
            case 0x4D1EEEE1:
                ROMFound=1;
                printf("Found Mac Plus ROM Version 1 'Lonely Hearts' *** Socket %d\n", Socket);
                break;

            case 0x4D1EEAE1:
                ROMFound=1;
                printf("Found Mac Plus ROM Version 2 'Lonely Heifers' *** Socket %d\n", Socket);
                break;

            case 0x4D1F8172:
                ROMFound=1;
                printf("Found Mac Plus ROM Version 3 'Loud Harmonicas' *** Socket %d\n", Socket);
                break;
        }
    }

    printf("\n");

    if(!ROMFound)
    {
        puts("No Mac Plus ROM sets found!");
        puts(" 1) Do you have a Gemulator ROM board installed?");
        puts(" 2) Did you specify the correct base address for your Gemulator ROM board?");
        puts(" 3) Is a Mac Plus ROM set properly installed on your Gemulator ROM board?");
    }
    else
    {
        puts("Gemulator ROM board settings for vMac.ini:");
        puts("(You can simply add the 3 lines below to the end of your vMac.ini file)\n");

        puts("CurrentROMPath = G");
        printf("GemulatorBase = %X\n", BaseAddr);
        puts("GemulatorSocket = (Set to socket number of Mac Plus ROM that you want to use)");
    }

    printf("\n");

    puts("Press ENTER to exit...");

    //Wait for ENTER key to be pressed...
    {
        char Temp;
        scanf("%c%c",&Temp,&Temp);
    }

    return;
}

/* Rearrange byte ordering in LONG's */
ULONG FlipLong (ULONG Data)
{
    ULONG New;

    New=(UBYTE) (Data >> 24);
    New|=((UBYTE) (Data >> 16)) << 8;
    New|=((UBYTE) (Data >> 8)) << 16;
    New|=((UBYTE) Data) << 24;

    return(New);
}


/*
  Much of the below code was written by Darek
  Michocka and adapted to work in this utility.
*/

#define cbSocket (128*1024)
#define countSocket (8)

unsigned char bigbuf[cbSocket*countSocket*2];

ULONG GetROMChecksum(UWORD Base, UBYTE Socket, UBYTE Length)
{
    int i;
    ULONG Checksum=0;

    _outp(Base,0);

    //UI has socket 1 as the first, but we use 0 as the first
    --Socket;

    if(Socket+Length > 8)
        return(0);

    for (i = 0; i < (Socket+Length)*cbSocket; i++)
        bigbuf[i] = _inp(Base);

    // scramble the bytes from socket order to byte order

    memmove(bigbuf + countSocket * cbSocket, bigbuf + Socket * cbSocket, Length * cbSocket);

    //Copy first 4 bytes, the checksum
    Checksum = (bigbuf[(countSocket + (0%Length)) * cbSocket + (0/Length)])
             | ((bigbuf[(countSocket + (1%Length)) * cbSocket + (1/Length)]) << 8)
             | ((bigbuf[(countSocket + (2%Length)) * cbSocket + (2/Length)]) << 16)
             | ((bigbuf[(countSocket + (3%Length)) * cbSocket + (3/Length)]) << 24) ;

    return (Checksum);
}